/*
	Listener.h
	Application Kit, Release 2.0
	Copyright (c) 1988, NeXT, Inc.  All rights reserved. 
*/

#import <Foundation/NSObject.h>
#import <mach/port.h>
#import <mach/message.h>
#import "NSPasteboard.h"

/*
 * Functions to enable/disable Services Menu items.  These should usually
 * only be called by service PROVIDERS (since they are the only ones who
 * know the name of the services, requestors don't).  The itemName in the
 * two functions below is the language-independent "Menu Item:" entry in
 * the __services section (which all provided services must have).  The
 * set function returns whether it was successful.
 * NXUpdateDynamicServices() causes the services information for the
 * system to be updated.  This will only be necessary if your program
 * adds dynamic services to the system (i.e. services not found in macho
 * segments of executables).
 */

extern BOOL NSShowsServicesMenuItem(NSString *itemName);
extern int NSSetShowsServicesMenuItem(NSString *itemName, BOOL enabled);
extern void NSUpdateDynamicServices(void);
extern BOOL NSPerformService(NSString *itemName, NSPasteboard *pboard);

/*
 * Names of workspace ports for requests and for acknowledging machlaunch
 * from the Workspace Manager.
 */

extern NSString *NXWorkspaceName;
extern NSString *NXWorkspaceReplyName;
#define NXWorkspaceName NXWorkspaceName
#define NXWorkspaceReplyName NXWorkspaceReplyName

/* reserved message numbers */

#define NSSelectorPMessage 35555
#define NSSelectorFMessage 35556
#define NSResponseMessage 35557
#define NSAcknowledge 35558

/* rpc return result error returns */

#define NSIncorrectMessage -20000

/* maximum number of remote method parameters allowed */

#define NSMaxRemoteMethodParameters 20

/* default timeouts in milliseconds */

#define NSDefaultSendAndReceiveTimeout 10000
#define NSDefaultSendAndReceiveTimeout 10000

#define NSMaxMessage	(2048-sizeof(msg_header_t)-\
			 sizeof(msg_type_t)-sizeof(int)-\
			 sizeof(msg_type_t)-8)
			 
typedef struct _NXMessage {	/* a message via mach */
    msg_header_t header;	/* every message has one of these */
    msg_type_t sequenceType;	/* sequence number type */
    int sequence;		/* sequence number */
    msg_type_t actionType;	/* selector string */
    char action[NSMaxMessage];
} NXMessage;

typedef struct _NXResponse {	/* a message via mach */
    msg_header_t header;	/* every message has one of these */
    msg_type_t sequenceType;	/* sequence number type */
    int sequence;		/* sequence number */
} NXResponse;

typedef struct _NXAcknowledge {	/* a message via mach */
    msg_header_t header;	/* every message has one of these */
    msg_type_t sequenceType;	/* sequence number type */
    int sequence;		/* sequence number */
    msg_type_t errorType;	/* error number type */
    int error;			/* error number, 0 is ok */
} NXAcknowledge;

typedef struct _NXRemoteMethod {/* defines method understood by Listener */
    SEL key;			/* obj-c selector */
    char *types;		/* defines types of parameters */
} NXRemoteMethod;

typedef union {			/* used to pass parameters to method */
    int ival;
    double dval;
    port_t pval;
    struct _bval {
        char *p;
        int len;
    } bval;
} NXParamValue;

extern char *NXCopyInputData(int parameter);
extern char *NXCopyOutputData(int parameter);
extern port_t NXPortFromName(NSString *portName, NSString *hostName);
extern port_t NXPortNameLookup(NSString *portName, NSString *hostName);
extern NXRemoteMethod *NXRemoteMethodFromSel(SEL s, NXRemoteMethod *pt);
extern id NXResponsibleDelegate(id self, SEL selector);

/*
 * permissible values of posType for setPosition:posType:andSelect:
 * and postion:posType:
 */

typedef enum _NSMsgPositionType {
    NSMsgTextPosition			= 0,
    NSMsgRegularExpressionPosition	= 1,
    NSMsgLineNumberPosition		= 2,
    NSMsgCharacterNumberPosition	= 3,
    NSMsgApplicationPosition		= 4
} NSMsgPositionType;

@interface Listener : NSObject
{
    char               *portName;
    port_t              listenPort;
    port_t              signaturePort;
    id                  delegate;
    int                 timeout;
    int                 priority;
    id                  _delegate2;
    id                  _requestDelegate;
    int                 _reservedListener2;
}
 
+ run;

- (int)timeout;
- setTimeout:(int)ms;
- (int)priority;
- setPriority:(int)level;
- (port_t)listenPort;
- (port_t)signaturePort;
- delegate;
- setDelegate:anObject;
- setServicesDelegate:anObject;
- servicesDelegate;
- (NSString *)portName;
- (int)checkInAs:(NSString *)name;
- (int)usePrivatePort;
- (int)checkOut;
- addPort;
- removePort;
- (NXRemoteMethod *)remoteMethodFor:(SEL)aSelector;
- messageReceived:(NXMessage *)msg;
- encodeWithCoder:(NSCoder *)aCoder;

- (int)msgQuit:(int *)flag;
- (int)msgCalc:(int *)flag;
- (int)msgDirectory:(char * const *)fullPath ok:(int *)flag;
- (int)msgVersion:(char * const *)aString ok:(int *)flag;
- (int)msgFile:(char * const *)fullPath ok:(int *)flag;
- (int)msgPrint:(const char *)fullPath ok:(int *)flag;
- (int)msgSelection:(char * const *)bytes length:(int *)len asType:(const char *)aType ok:(int *)flag;
- (int)msgSetPosition:(const char *)aString posType:(NSMsgPositionType)aPosition andSelect:(int)sflag ok:(int *)flag;
- (int)msgPosition:(char * const *)aString posType:(int *)anInt ok:(int *)flag;
- (int)msgCopyAsType:(const char *)aType ok:(int *)flag;
- (int)msgCutAsType:(const char *)aType ok:(int *)flag;
- (int)msgPaste:(int *)flag;
- (int)performRemoteMethod:(NXRemoteMethod *)method paramList:(NXParamValue *)params;

@end
